
#=============================================================================
#				PARAMETER DEFINITIONS
#-----------------------------------------------------------------------------

#	m	Rank of event value from top
#	M	Number of ranks used from top = R for epoch methods
#	P	Cumulative probability
#	r	Rate of events in epoch (rate/year)
#	R	Record length in epochs (years)
#	T	Number of bootstrap trials
#	V	Event values
#	y	Reduced variates

#   method	"GRG"	Gringorten-Cook-Harris wLMS	(epoch)
#		"GUM" Gumbel LMS				(epoch)
#		"MLE"	Gumbel MLE				(epoch)
#		"MOM"	Method of Moments			(epoch)
#		"GRW" GRG free fit of w index		(epoch)
#		"GEV" GEV wLMS				(epoch)
#		"XMS"	XIMIS					(peaks over threshold)
#		"POT"	POT by MLE				(peaks over threshold)
#		"XMW" XIMIS free fit of w index	(peaks over threshold)


#library(ismev)	# maximum likelihood estimation
library(evir)

#Method metadata
method.meta=data.frame(name=c("Gringorten-Cook-Harris","Least Mean Squares",
   "Maximum Likelihood Estimation","Method of Moments","XIMIS","POT by MLE"),
 type=c(rep("epoch",4),rep("pot",2)))
rownames(method.meta)<-c("GRG","GUM","MLE","MOM","XMS","POT")                     


#===================================================================================
# 					FUNCTIONS FOR EVA
#-----------------------------------------------------------------------------------

	# FIT EPOCH MAXIMA
	# ----------------
EVA.epoch.fit=function(V,method=c("GRG","MLE","GUM","MOM","GRW","GEV"),w=1)
{mx<-matrix(0,ncol=3,nrow=length(method))
 colnames(mx)<-c("Mode","Dispersion","Index")
 rownames(mx)<-method
 for (i in c(1:length(method)))
 {# Return c(U,D,w) for qximis()
  if(method[i]=="GUM"){mx[i,]<-c(summary(epoch.GUM(V,w))$coefficients[1:2]^(1/w),w)}
  if(method[i]=="GRG"){mx[i,]<-c(summary(epoch.GRG(V,w))$coefficients[1:2]^(1/w),w)}
  if(method[i]=="MLE"){mx[i,]<-c(as.numeric(epoch.MLE(V,w))^(1/w),w)}
  if(method[i]=="MOM"){mx[i,]<-c(epoch.MOM(V,w)[1:2]^(1/w),w)}
  if(method[i]=="GRW"){par<-epoch.GRW(V)$par;mx[i,]<-c(par[1:2]^(1/w),par[3]) }
  # Returns c(mu,sig,xi) for qgev()
  if(method[i]=="GEV"){mx[i,]<-epoch.GEV(V)$par}
 }
 as.data.frame(mx)
}

EVA.pot.fit=function(V,R,method=c("XMS","MLE","XMW"),w=1)
{mx<-matrix(0,ncol=3,nrow=length(method))
 colnames(mx)<-c("Mode","Dispersion","Index")
 rownames(mx)<-method
 for (i in c(1:length(method)))
 {if(method[i]=="XMS"){mx[i,]<-c(summary(pot.XMS(V,R,w))$coefficients[1:2]^(1/w),w)}
  if(method[i]=="MLE"){mx[i,]<-c(pot.MLE(V,R,w)^(1/w),w)}
  if(method[i]=="XMW"){mx[i,]<-pot.XMW(V,R)}
 }
 as.data.frame(mx) 
}
 
EVA.confidence.data=function(P=c(0.05,0.95),M=100,C=c("median","mean"))
		#	0.005 <= P <= 0.995 *** M <= 100
{m<-c(1:M[1])
 if (C[1]=="mean")
  {Y=function(P,m){sapply(P,qgumb,m=m)-ygumb.mean(m)} # centred on mean
  }else{
   Y=function(P,m){sapply(P,qgumb,m=m)-qgumb(0.5,m)}  # centred on median
  }
 L<-sapply(m,Y,P=P)
 colnames(L)<-paste("m",m,sep="")
 rownames(L)<-paste("P",P*100,"%",sep="")
 t(L)
}

EVA.stderr=function(y,M,method=c("XMS","GRG","MLE","GUM","MOM"))
{fn<-paste("prediction.variance.",method[1],sep="")
 sqrt(do.call(fn,list(y=y,M=M)))
}

EVA.confidence.prediction=function(P=c(0.05,0.95),y,M,method=c("XMS","POT","GRG","MLE","GUM","MOM"))
{se<-EVA.stderr(y,M,method[1])
 normP<-qnorm(P)
 Y<-sapply(normP,function(x){x*se})
 colnames(Y)<-round(P,4)
 rownames(Y)<-round(y,4)
 Y
}
 



#------------------------------------------------------------------------------
# 	Constants from bootstap calibrations in EVA.calbrate.R & EVA.bootstrap.xlsx
#------------------------------------------------------------------------------

	# Least Mean Squares 
GUM.mode.bias=c(0.03560584,-0.031271028,0.007873606,-0.000795976,2.86235E-05)
GUM.dispersion.bias=-0.7065051
GUM.mode.variance=1.154389		# /M
GUM.dispersion.variance=1.127671	# /M

GUM.covariance=c( 0.222410737,-1.0995560044 )
GUM.covariance=c(0.2117,-1.0925)	# a*M^b

	# Method of Moments
MOM.mode.bias=c(-0.183930026,0.216301242,-0.100016444,0.023491403,-0.002979494,
         0.000194715,-5.14799E-06)
MOM.dispersion.bias=-0.5337562	# /M +1
MOM.mode.variance=1.641709000548 	# /M
MOM.dispersion.variance=1.057380451	# /M
MOM.covariance=0.71983865		# /M

	# Gringorten-Cook-Harris
GRG.mode.bias=c(-0.235030384,0.232443338,-0.091896732,0.018847599,
				-0.002129876,0.000126255,-3.07511E-06)
GRG.dispersion.bias=c(0.000125954,0.144323066,-0.245653376)
GRG.mode.variance=1.14742003019089	# /M
GRG.dispersion.variance=0.72045396	# /M
GRG.covariance=0.227801825		# /M

	# Maximim Likelihood Estimation
MLE.mode.bias=c(0.247249823,-0.16891127,0.04694895,-0.006621647,0.000478809,
		-1.51244E-05,9.20948E-08)
MLE.dispersion.bias=-0.7930795
MLE.mode.variance=1.11746916035405 
MLE.dispersion.variance=0.60946106
MLE.covariance=0.252307685

	# XIMIS wLMS
XMS.mode.bias=c(0.109971496,-0.118240372,0.05285166,-0.01234504,
			0.001579528,-0.000104874,2.82648E-06)
XMS.dispersion.bias=0.0054431
XMS.mode.variance=1.08508264459637 
XMS.dispersion.variance=1.186679744
XMS.covariance=-0.15108421

	# POT MLE
POT.mode.bias=c(0.3017333707,-0.175730428,0.0350686917,
        -0.0015288407,-0.0003808099,5.21559e-05,-1.933e-06) 
POT.dispersion.bias=-1.05949836164154 
POT.mode.variance=0.909286926406287  
POT.dispersion.variance=0.978497826908846 
POT.covariance=0.000108269047642685 

#------------------------------------------------------------------------------
#	Probability density functions	(m-th highest extremes) 
#------------------------------------------------------------------------------

	# Gumbel FT1
dgumb=function(y,m=1) 	# from Gumbel equation 5.3.1(2)
 {m[1]^m[1]/factorial(m[1]-1)*exp(-m[1]*y-m[1]*exp(-y))}

#------------------------------------------------------------------------------
#	Cumulative distribution functions	(m-th highest extremes) 
#------------------------------------------------------------------------------

	# Gumbel FT1
pgumb=function(y,m=1)	# from Gumbel equation 5.3.2(3)
 {P<-exp(-exp(-y))^m[1]
  S<-0
  for (v in c(0:(m[1]-1))) S<-S+m[1]^v*exp(-v*y)/factorial(v)
  P*S
 }

#------------------------------------------------------------------------------
#	Quantile functions	(m-th highest extremes) 
#------------------------------------------------------------------------------

	# Gumbel FT1
qgumb=function(P,m=1)	# if (m>1) then 1E-5 <= P <= 0.0.9999997 and m <= 100
{if (m>1)
 {qg=function(P=0.5,m=1)
  {#errfn<-function(y,P,m=1) {(P-pgumb(y[1],m[1]))^2}
   #a<-optimize(errfn,c(-2,5),P=P[1],m=m[1])$minimum
   fn<-function(y,P,m) {pgumb(y[1],m[1])-P[1]}
   b<-uniroot(fn,c(-2.4,15),P=P[1],m=m[1])$root
   b
  }
  if(min(P)<1E-5) stop("P value too small")
  if(max(P)>0.9999997) stop("P value too large")
  if (m>100) stop("m > 100") 
  z<-sapply(P,qg,m=m)
 }else{z<- -log(-log(P))}
 z 
}

#qximis=function(y,U,D,w=1) {(U^w+y*D^w)^(1/w)}
qximis=function(y,pot) {(pot[1]^pot[3]+y*pot[2]^pot[3])^(1/pot[3])}

qgev=function(gev,P){gev[1]+gev[2]/gev[3]*((-log(P))^-gev[3]-1)}

#------------------------------------------------------------------------------
#	Random generation functions	 
#------------------------------------------------------------------------------

	# Gumbel FT1
rgumb=function(R,U=0,D=1,w=1) {(U^w-log(-log(runif(R)))*D^w)^(1/w)}

	# XIMIS 	left-censored samples from exponential parent
rximis=function(M,R=1,U=0,D=1,w=1,r=10)		
 {P<-runif(M*r)
  P<-P[rank(P)<=M]
  (U^w+D^w*(-log(P)-log(r*M/R)))^(1/w)
  #V<- -log(runif(M*r))
  #V<-sort(V,decreasing=TRUE)
  #U+(V[1:M]-log(r*M/R))*D
 }


#------------------------------------------------------------------------------
#	Reduced variate functions
#------------------------------------------------------------------------------
#		*** ALL RANKED DOWNWARDS FROM TOP ***
 
y.gumb=function(R){rev(-log(-log(c(1:R)/(R+1))))}

y.gring.mean=function(R)
 {A<-0.43854-0.46617/log(R)
  B<-0.448
  rev(-log(-log((c(1:R)-A)/(R+1-A-B))))
 }
y.gring.var=function(R)
 {y<-y.gring.mean(R)
  a0<-1.279; a1<-0.235; a2<-0.334; a3<- -0.0541; a4<-0.0121
  (a0+a1*y+a2*y^2+a3*y^3+a4*y^4)^2/R
 }

y.ximis.mean=function(M,R){yximis(M,R)$mean}
y.ximis.variance=function(M,R){yximis(M,R)$var}
	# empirical rank equivalent to y
rank.ximis=function(y,R) {exp(-0.9474*(yy-log(R))+0.2172)}



#------------------------------------------------------------------------------
#	Distribution fitting functions	 
#------------------------------------------------------------------------------

	# Least Mean Squares (Gumbel, unweighted)
epoch.GUM=function(V,w=1)		
{x<-sort(V^w)
 n<-length(x)
 y<-qgumb(c(1:n)/(n+1))
 lm(x~y)
}
	# Method of Moments
epoch.MOM=function(V,w=1)
{mx<-mean(V^w); sx<-sd(V^w)
 D<-sx*sqrt(6)/pi
 z<-c(mx+digamma(1)*D,D)
 names(z)<-c("Umom","Dmom") 
 z
}

epoch.MLE=function(V,w=1)	
{#gum.fit(V^w)$mle[1:2]			# WRAPPER for ismev function
 gumbel(V^w)$par.ests[c(2,1)]		# WRAPPER for evir function
}


	# Gringorten-Cook-Harris
epoch.GRG=function(V,w=1)
{x<-sort(V^w)
 n<-length(x)
 A<-0.43854-0.46617/log(n)
 B<-0.448
 y<- -log(-log((c(1:n)-A)/(n+1-A-B)))
 a0<-1.279; a1<-0.235; a2<-0.334; a3<- -0.0541; a4<-0.0121
 var<-(a0+a1*y+a2*y^2+a3*y^3+a4*y^4)^2/n
 lm(x~y,weights=1/var)
}

	# Gringorten-Cook-Harris with free fit of Weibull index w
epoch.GRW=function(V,interval=c(0,5))
{R<-length(V)
 y<-y.gring.mean(R)
 var<-y.gring.var(R)
 x<-c(summary(epoch.GRG(V,1))$coefficients[1:2],1)
  errfn<-function(x){sum((V-qximis(y,x))^2/var)}
 optim(x,errfn)
}

if (FALSE)
{{
epoch.GRW=function(V,interval=c(0,5))
{R<-length(V)
 y<-y.gring.mean(R)
 var<-y.gring.var(R)
 fn<-function(w){grr<-c(summary(epoch.GRG(V,w))$coefficients[1:2]^(1/w),w)
                 sum((V-qximis(y,grr))^2/var)}
 options(warn=-1)
 w<-optimize(fn,interval=interval)$minimum 
 options(warn=0)
 if ((w-interval[1])<0.01) warning("epoch.GRW, w close to lower")
 if ((interval[2]-w)<0.01) warning("epoch.GRW, w close to upper")
 c(summary(epoch.GRG(V,w))$coefficients[1:2]^(1/w),w)
}

epoch.GRW=function(V)
{fn<-function(w){-summary(epoch.GRG(V,w))$r.squared}
 w<-optimize(fn,lower=0,upper=20)$minimum 
 wX<-epoch.GRG(V,w)
 c(summary(wX)$coefficients[1:2]^(1/w),w)
}
}}

epoch.GEV=function(V)
{R<-length(V)
 y<-y.gring.mean(R)
 P<-exp(-exp(-y))
 var<-y.gring.var(R)
 sig<-sqrt(6*var(V))/pi
 mu<-mean(V)-0.57722*sig
 x<-c(mu,sig,0.1)
  errfn<-function(x){sum((V-qgev(x,P))^2/var)}
 optim(x,errfn)
}



	# Exponential distribution by Least Mean Squares	!!!!!!!!!!!!
fit.exponentialLMS=function(V)		
{x<-sort(V)
 n<-length(x)
 y<- -log((1-c(1:n)/(n+1)))
 lm(x~y)
}
	# XIMIS by weighted MLS
pot.XMS=function(V,R=length(V),w=1)
{x<-sort(V^w,decreasing=TRUE)
 y<-yximis(length(x),R)
 my<-y$mean
 wt<-1/y$var
 lm(x~my,weights=wt)
}

	# XIMIS by weighted MLS with free fit of Weibull index w
pot.XMW=function(V,R=length(V))
{M<-length(V)
 y<-y.ximis.mean(M,R)
 var<-y.ximis.variance(M,R)
 x<-c(summary(pot.XMS(V,R,1))$coefficients[1:2],1)
  errfn<-function(x){sum((V-qximis(y,x))^2/var)}
 optim(x,errfn)
}

pot.MLE=function(V,R=1,w=1)	
{M<-length(V)
 x<-V^w
 gD<-mean(x-min(x))
 yR<-log(R)-digamma(1)
# pR<-exp(-exp(-yR))
 gU<-min(x)+log(M/R)*gD
 c(gU,gD)
}

#------------------------------------------------------------------------------
#	Bias, variance and covariance of fitted mode and dispersion 	 
#------------------------------------------------------------------------------

	# Least Mean Squares (Gumbel, unweighted)
bias.mode.GUM=function(R)
  {sapply(R,function(x)
   {if (x<10000){sum(GUM.mode.bias*log(x)^c(0:4))}else{0}}
   )}
bias.dispersion.GUM=function(R){1+R^GUM.dispersion.bias}
variance.mode.GUM=function(R){GUM.mode.variance/R}
variance.dispersion.GUM=function(R){GUM.dispersion.variance/R}
covariance.GUM=function(R){GUM.covariance[1]*R^GUM.covariance[2]}

	# Method of Moments
bias.mode.MOM=function(R) 
  {sapply(R,function(x)
   {if (x<2000){sum(MOM.mode.bias*log(x)^c(0:6))}else{0}}
   )}
bias.dispersion.MOM=function(R){1+MOM.dispersion.bias/R}
variance.mode.MOM=function(R) {MOM.mode.variance/R}
variance.dispersion.MOM=function(R){MOM.dispersion.variance/R}
covariance.MOM=function(R){MOM.covariance/R}

	# Gringorten-Cook-Harris
bias.mode.GRG=function(R)
  {sapply(R,function(x)
   {if (x<10000){sum(GRG.mode.bias*log(x)^c(0:6))}else{0}}
   )}
bias.dispersion.GRG=function(R)
  {sapply(R,function(x)
   {if (x<10000){sum(GRG.dispersion.bias*(1/x)^c(0:2))+1}else{1}}
   )}
variance.mode.GRG=function(R){GRG.mode.variance/R}
variance.dispersion.GRG=function(R){GRG.dispersion.variance/R}
covariance.GRG=function(R){GRG.covariance/R}

	# Maximum Likelihood Estimation
bias.mode.MLE=function(R)
  {sapply(R,function(x)
   {if (x<10000){sum(MLE.mode.bias*log(x)^c(0:6))}else{0}}
   )}
bias.dispersion.MLE=function(R){1+MLE.dispersion.bias/R}
variance.mode.MLE=function(R) {MLE.mode.variance/R}
variance.dispersion.MLE=function(R){MLE.dispersion.variance/R}
covariance.MLE=function(R){MLE.covariance/R}

	# XIMIS
bias.mode.XMS=function(M)
  {sapply(M,function(x)
   {if (x<10000){sum(XMS.mode.bias*log(x)^c(0:6))}else{0}}
   )}
bias.dispersion.XMS=function(M){1+XMS.dispersion.bias/M}
variance.mode.XMS=function(M) {XMS.mode.variance/M}
variance.dispersion.XMS=function(M){XMS.dispersion.variance/M}
covariance.XMS=function(M){XMS.covariance/M}

	#POT MLE
bias.mode.POT=function(R)
  {sapply(R,function(x)
   {if (x<10000){sum(POT.mode.bias*log(x)^c(0:6))}else{0}}
   )}
bias.dispersion.POT=function(R){1+POT.dispersion.bias/R}
variance.mode.POT=function(R) {POT.mode.variance/R}
variance.dispersion.POT=function(R){POT.dispersion.variance/R}
covariance.POT=function(R){POT.covariance/R}

#------------------------------------------------------------------------------
#	Prediction accuracy at y for n observations 	 
#------------------------------------------------------------------------------

prediction.variance.gumbel=function(y,M){(1+(1+digamma(1)+y)^2)/M}
prediction.variance.GUM=function(y,M)
   {variance.mode.GUM(M)+2*y*covariance.GUM(M)+y^2*variance.dispersion.GUM(M)}
prediction.variance.MOM=function(y,M)
   {variance.mode.MOM(M)+2*y*covariance.MOM(M)+y^2*variance.dispersion.MOM(M)}
prediction.variance.MLE=function(y,M)
   {variance.mode.MLE(M)+2*y*covariance.MLE(M)+y^2*variance.dispersion.MLE(M)}
prediction.variance.GRG=function(y,M)
   {variance.mode.GRG(M)+2*y*covariance.GRG(M)+y^2*variance.dispersion.GRG(M)}
prediction.variance.XMS=function(y,M)
   {variance.mode.XMS(M)+2*y*covariance.XMS(M)+y^2*variance.dispersion.XMS(M)}
prediction.variance.POT=function(y,M)
   {variance.mode.POT(M)+2*y*covariance.POT(M)+y^2*variance.dispersion.POT(M)}

prediction.bias.GUM=function(y,M)
	{bias.mode.GUM(M)+y*(bias.dispersion.GUM(M)-1)}
prediction.bias.MOM=function(y,M)
	{bias.mode.MOM(M)+y*(bias.dispersion.MOM(M)-1)}
prediction.bias.MLE=function(y,M)
	{bias.mode.MLE(M)+y*(bias.dispersion.MLE(M)-1)}
prediction.bias.GRG=function(y,M)
	{bias.mode.GRG(R)+y*(bias.dispersion.GRG(M)-1)}
prediction.bias.XMS=function(y,M)
	{bias.mode.XMS(M)+y*(bias.dispersion.XMS(M)-1)}
prediction.bias.POT=function(y,M)
	{bias.mode.POT(M)+y*(bias.dispersion.POT(M)-1)}




#------------------------------------------------------------------------------
#	Bootstrap fitting functions. 	 T trials of M ranks
#------------------------------------------------------------------------------

# Mode and dispersion of every trial
# ----------------------------------
	# Least Mean Squares (Gumbel, unweighted)
bootstrap.GUM=function(M,T=1E5) 
{z<-sapply(c(1:T),function(i) {summary(epoch.GUM(rgumb(M)))$coefficients[1:2]}) 
 data.frame(Ulgum=z[1,],Dgum=z[2,])
}
	# Method of Moments
bootstrap.MOM=function(M,T=1E5)
{g<- -digamma(1); f<-sqrt(6)/pi
 z<-sapply(c(1:T),function(i) {x<-rgumb(M); mx<-mean(x); sx<-sd(x)
 c(mx-g,sx*f)}) 
 data.frame(Umom=z[1,],Dmom=z[2,])
}
	# Maximum Likelihood Estimation
bootstrap.MLE=function(M,T=1E5)
{z<-sapply(c(1:T),function(i) {epoch.MLE(rgumb(M))}) 
 data.frame(Umle=z[1,],Dmle=z[2,])
}
	# Gringorten-Cook-Harris
bootstrap.GRG=function(M,T=1E5)
{z<-sapply(c(1:T),function(i)
  {summary(epoch.GRG(rgumb(M)))$coefficients[1:2]
  }) 
 data.frame(Ugrg=z[1,],Dgrg=z[2,])
}
	# XIMIS by weighted MLS
bootstrap.XMS=function(M,R=M,T=1E5,r=10)
{z<-sapply(c(1:T),function(i)
  {x<-rximis(M,R,r=r)
   x<-sort(x,decreasing=TRUE)
   x<-x[1:M]
   summary(pot.XMS(x,R=R))$coefficients[1:2]
  }) 
 data.frame(Uxms=z[1,],Dxms=z[2,])
}
	# POT by MLE
bootstrap.POT=function(M,R=M,T=1E5,r=10)
{z<-sapply(c(1:T),function(i)
  {x<-rximis(M,R,r=r)
   x<-sort(x,decreasing=TRUE)
   x<-x[1:M]
   pot.MLE(x,R=R)
  }) 
 data.frame(Upot=z[1,],Dpot=z[2,])
}

	# Exponential distribution, Least Mean Squares 
bootstrap.exponentialLMS=function(M,T=1E5) # Bootstrap T trials of M values
{z<-sapply(c(1:T),function(i) {summary(fit.exponentialLMS(rexp(M)))$coefficients[1:2]}) 
 data.frame(Uexpon=z[1,],Lexpon=z[2,])
}

# Mode and dispersion, means variances and covariance
# ---------------------------------------------------
	# Least Mean Squares (Gumbel, unweighted)
bootstrap.GUM.moments=function(M,T=1E5)	
{z<-bootstrap.GUM(M,T)
 list(meanUgum=mean(z$Ugum),meanDgum=mean(z$Dgum),varUgum=var(z$Ugum),
      varDgum=var(z$Dgum),covUDgum=cov(z$Ugum,z$Dgum))
}
	# Method of Moments
bootstrap.MOM.moments=function(M,T=1E5)	
{z<-bootstrap.MOM(M,T)
 list(meanUmom=mean(z$Umom),meanDmom=mean(z$Dmom),varUmom=var(z$Umom),
      varDmom=var(z$Dmom),covUDmom=cov(z$Umom,z$Dmom))
}
	# Maximum Likelihood Estimation
bootstrap.MLE.moments=function(M,T=1E5)	
{z<-bootstrap.MLE(M,T)
 list(meanUmle=mean(z$Umle),meanDmle=mean(z$Dmle),varUmle=var(z$Umle),
      varDmle=var(z$Dmle),covUDmle=cov(z$Umle,z$Dmle))
}
	# Gringorten-Cook-Harris
bootstrap.GRG.moments=function(M,T=1E5)
{z<-bootstrap.GRG(M,T)
 list(meanUgrg=mean(z$Ugrg),meanDgrg=mean(z$Dgrg),varUgrg=var(z$Ugrg),
      varDgrg=var(z$Dgrg),covUDgrg=cov(z$Ugrg,z$Dgrg))
}
	# XIMIS by weighted MLS
bootstrap.XMS.moments=function(M,R=M,T=1E5,r=10)
{z<-bootstrap.XMS(M,R,T,r)
 list(meanUxms=mean(z$Uxms),meanDxms=mean(z$Dxms),varUxms=var(z$Uxms),
      varDxms=var(z$Dxms),covUDxms=cov(z$Uxms,z$Dxms))
}
	# POT by MLE
bootstrap.POT.moments=function(M,R=M,T=1E5,r=10)
{z<-bootstrap.POT(M,R,T,r)
 list(meanUpot=mean(z$Upot),meanDpot=mean(z$Dpot),varUpot=var(z$Upot),
      varDpot=var(z$Dpot),covUDpot=cov(z$Upot,z$Dpot))
}	
# Exponential distribution, Least Mean Squares 
bootstrap.exponentialLMS.moments=function(M,T=1E5)	
{z<-bootstrap.exponentialLMS(M,T)
 list(meanUexpon=mean(z$Uexpon),meanLexpon=mean(z$Lexpon),varUexpon=var(z$Uexpon),
      varLexpon=var(z$Lexpon),covULexpon=cov(z$Uexpon,z$Lexpon))
}


#------------------------------------------------------------------------------
#	XIMIS utility functions	 
#------------------------------------------------------------------------------

	# Return mean y of m-th highest extremes by integrating pdf 
ygumb.mean<-function(m=c(1:10))
{F=function(m){integrate(function(y,m){dgumb(y,m)*y},-Inf,Inf,m)$value}
  sapply(m,F)
}
	# Return data.frame with XIMIS mean and var of y for top N ranks
	# from Gumbel equation 5.3.3.(4)
yximis=function(M=100,R=1)
{y<-rep(0,M[1]); var<-y
 y[1]<- -digamma(1)+log(R[1]); var[1]<-pi^2/6		# Gumbel 4.2.1(11)&(13)
 for(i in c(2:M)){y[i]<-y[i-1]-1/(i-1); var[i]<-var[i-1]-1/(i-1)^2}  # (10)&(12)
 data.frame(mean=y,var=var)
}
